-------------------------------------------------------------------------------
-- MrTrader 
--
-- Suite of tools for the avid crafter.
--
-- Send suggestions, comments, and bugs to user@zedonline.net
-------------------------------------------------------------------------------

MrTraderSkillWindow = {};

local NUM_FILTERS_TO_DISPLAY = 18;
local TRADE_SKILLS_DISPLAYED = 11;
local TRADE_SKILL_TEXT_WIDTH = 275;
MRTRADER_TRADE_SKILL_HEIGHT = 16;

MRTRADER_TRADE_SKILL_FILTER_HEIGHT = 20;

-- Coloring for trade skills
local TradeSkillTypeColor = { };
TradeSkillTypeColor["optimal"]	= { r = 1.00, g = 0.50, b = 0.25,	font = GameFontNormalLeftOrange };
TradeSkillTypeColor["medium"]	= { r = 1.00, g = 1.00, b = 0.00,	font = GameFontNormalLeftYellow };
TradeSkillTypeColor["easy"]		= { r = 0.25, g = 0.75, b = 0.25,	font = GameFontNormalLeftLightGreen };
TradeSkillTypeColor["trivial"]	= { r = 0.50, g = 0.50, b = 0.50,	font = GameFontNormalLeftGrey };
TradeSkillTypeColor["header"]	= { r = 1.00, g = 0.82, b = 0,		font = GameFontNormalLeft };

-- Colorblind Support
local TradeSkillTypePrefix = {
["optimal"] = " [+++] ",
["medium"] = " [++] ",
["easy"] = " [+] ",
["trivial"] = " ", 
["header"] = " ",
}

UIPanelWindows["MrTrader_SkillFrame"] =	{ area = "left", pushable = 3 };

local L = LibStub("AceLocale-3.0"):GetLocale("MrTrader", true);
local Menu = LibStub("Dewdrop-2.0");

-----
-- XML Definitions
-----
MRTRADER_NEW_CATEGORY = L["New Group"];
MRTRADER_NEW_FAVORITE_GROUP = L["New Favorite Group"];
MRTRADER_NAME = L["Name"];

-----
-- New Category Dialog
-----
function MrTrader_NewCategoryFrame_Cancel()
	MrTrader_NewCategoryFrame:Hide();
end

function MrTrader_NewCategoryFrame_Okay()
	local categoryName = MrTrader_NewCategoryFrameCategoryName:GetText();

	if( categoryName == "" or categoryName == nil ) then
		return;
	end

	local tradeskillSpellID = MrTrader:MatchPartialCraftSkillName(GetTradeSkillLine());
	local favorites = MrTrader:GetFavoritesForSkill(tradeskillSpellID);
	
	if( favorites ~= nil and favorites[categoryName] == nil ) then
		MrTrader:AddFavoriteCategoryToSkill(tradeskillSpellID, categoryName);

		if( MrTrader_NewCategoryFrame.referenceSkill ) then
			-- Set the reference skill up in the new category			
			local skillSpellID = MrTrader:GetSpellIDForRecipe(GetTradeSkillRecipeLink(MrTrader_NewCategoryFrame.referenceSkill));
			MrTrader:AddFavoriteToSkill(tradeskillSpellID, categoryName, skillSpellID);
		end
	end
	
	MrTraderSkillWindow.PopulateFilterTree();
	MrTraderSkillWindow.Update();
	MrTrader_NewCategoryFrame:Hide();
end

-----
-- Context Menus
-----
local MenuSpacer = {
	text = "",
	isTitle = true,
}

function MrTraderSkillWindow:DisplaySkillContext(frame, skillIndex)
	Menu:Open(frame, "children", function() MrTraderSkillWindow:UpdateSkillContext(skillIndex); end);
end

function MrTraderSkillWindow:UpdateSkillContext(skillIndex)
	menu = {};
	local index = 100;
	local _, _, numAvailable, _, altVerb = GetTradeSkillInfo(skillIndex);
	
	if( not IsTradeSkillLinked() ) then
		menu[index] = {
			text = altVerb or CREATE,
			closeWhenClicked = true,
			func = function()
				if ( (not PartialPlayTime()) and (not NoPlayTime()) ) then
					DoTradeSkill(skillIndex, 1);
					MrTrader_SkillInputBox:ClearFocus();
				end
			   end,
		}
		index = index + 1;
	
		if( not altVerb ) then
			menu[index] = {
				text = CREATE_ALL,
				closeWhenClicked = true,
				func = function()
					if ( (not PartialPlayTime()) and (not NoPlayTime()) ) then
						MrTrader_SkillInputBox:SetNumber(numAvailable);
						DoTradeSkill(skillIndex, numAvailable);
						MrTrader_SkillInputBox:ClearFocus();
					end
			   	end,
			}
			index = index + 1;
		end
	
		menu[index] = MenuSpacer;
		index = index + 1;
	end
	
	menu[index] = {
		text = L["Favorite Groups"],
		textR = 1.0,
		textG = 0.8,
		textB = 0,
		isTitle = true,
	}
	index = index + 1;
	
	local skillSpellID = MrTrader:MatchPartialCraftSkillName(GetTradeSkillLine());
	local favorites = MrTrader:GetFavoritesForSkill(skillSpellID);
	
	if( favorites ) then
		for categoryName, categoryItems in pairs(favorites) do
			local skillSpell = MrTrader:GetSpellIDForRecipe(GetTradeSkillRecipeLink(skillIndex));
			local isIncluded = categoryItems[skillSpell] ~= nil;
			menu[index] = {
				text = categoryName,
				closeWhenClicked = false,
				checked = isIncluded,
				func = function ()
					if(not isIncluded) then
						MrTrader:AddFavoriteToSkill(skillSpellID, categoryName, skillSpell);
					else
						MrTrader:DelFavoriteFromSkill(skillSpellID, categoryName, skillSpell);
					end
					
					MrTraderSkillWindow.Update();
					isIncluded = not isIncluded;
					menu[index].checked = isIncluded;
				end,
			}
			index = index + 1;
		end
	end

	menu[index] = {
		text = L["Add to New Favorite Group..."],
		closeWhenClicked = true,
		func = function ()
			MrTrader_NewCategoryFrame.referenceSkill = skillIndex;
			MrTrader_NewCategoryFrame:Show();
		end,
	}
	index = index + 1;
	
	menu[index] = MenuSpacer;
	index = index + 1;
	
	menu[index] = {
		text = L["Close"],
		closeWhenClicked = true,
	}
	
	Menu:FeedTable(menu);
end

function MrTraderSkillWindow:DisplayFilterContext(frame)
	local filter = frame.filterRef;
	
	if( filter ) then
		Menu:Open(frame, "children", function() MrTraderSkillWindow:UpdateFilterContext(filter); end);
	end
end

function MrTraderSkillWindow:UpdateFilterContext(filter)
	menu = {};
	local index = 100;
	
	if( filter.filterType == "favorite" and filter.filterIndex == "__all" ) then
		menu[index] = {
			text = L["New Favorite Group"],
			closeWhenClicked = true,
			func = function()
				MrTrader_NewCategoryFrame.referenceSkill = nil;
				MrTrader_NewCategoryFrame:Show();
			end,
		}
		index = index + 1;
	elseif( filter.filterType == "favorite" ) then
		menu[index] = {
			text = L["Remove Favorite Group"],
			closeWhenClicked = true,
			func = function()
				MrTrader:DelFavoriteCategoryFromSkill(MrTrader:MatchPartialCraftSkillName(GetTradeSkillLine()), filter.filterIndex);
				
				MrTraderSkillWindow.PopulateFilterTree();
				MrTraderSkillWindow.Update();
			end,
		}
		index = index + 1;
	end
	
	if( index ~= 100 ) then
		menu[index] = MenuSpacer;
		index = index + 1;
	end
	
	menu[index] = {
		text = L["Close"],
		closeWhenClicked = true,
	}
	
	Menu:FeedTable(menu);
end

-----
-- Tradeskill Window
-----
function MrTraderSkillWindow.Show()
	MrTraderSkillWindow.filters = {};
	ShowUIPanel(MrTrader_SkillFrame);
	SetPortraitTexture(MrTrader_SkillFramePortrait, "player");
	
	FauxScrollFrame_SetOffset(MrTrader_SkillListScrollFrame, 0);
	MrTrader_SkillListScrollFrameScrollBar:SetMinMaxValues(0, 0); 
	MrTrader_SkillListScrollFrameScrollBar:SetValue(0);
	
	MrTraderSkillWindow.filterCategory = nil;
	MrTraderSkillWindow.filterSelection = nil;
	MrTraderSkillWindow.PopulateFilterTree();
	
	MrTraderSkillWindow_OnSearchTextChange(MrTrader_SkillFrameEditBox);
	TradeSkillOnlyShowMakeable(MrTrader_SkillFrameAvailableFilterCheckButton:GetChecked());
	
	MrTraderSkillWindow.SetSkillSelection(nil, 0);	
	MrTraderSkillWindow.Update();
end

function MrTraderSkillWindow.PopulateFilterTree()
	MrTraderSkillWindow.filters = {};
	
	tinsert(MrTraderSkillWindow.filters, 
		{
			title = L["All Items"],
			filterType = "all",
			subgroups = {},
		});
	tinsert(MrTraderSkillWindow.filters, 
		{
			title = L["Categories"],
			subgroups = {},
		});
	tinsert(MrTraderSkillWindow.filters, 
		{
			title = L["Inventory Slots"],
			subgroups = {},
		});
		
	local groups = {GetTradeSkillSubClasses()};
	local numGroups = getn(groups);
	local parent = MrTraderSkillWindow.filters[2];
	for i=1, numGroups do
		tinsert(parent.subgroups, 
			{
				title = groups[i],
				filterType = "category",
				filterIndex = i,
			});
	end
	
	groups = {GetTradeSkillInvSlots()};
	numGroups = getn(groups);
	local parent = MrTraderSkillWindow.filters[3];
	for i=1, numGroups do
		tinsert(parent.subgroups, 
			{
				title = groups[i],
				filterType = "invslot",
				filterIndex = i,
			});
	end	
	
	tinsert(MrTraderSkillWindow.filters, 
		{
			title = L["Difficulty"],
			subgroups = {},
		});
	parent = MrTraderSkillWindow.filters[4];
	tinsert(parent.subgroups, 
		{
			title = L["Optimal"],
			filterType = "difficulty",
			filterIndex = "optimal",
		});
	tinsert(parent.subgroups, 
		{
			title = L["Medium"],
			filterType = "difficulty",
			filterIndex = "medium",
		});
	tinsert(parent.subgroups, 
		{
			title = L["Easy"],
			filterType = "difficulty",
			filterIndex = "easy",
		});
	tinsert(parent.subgroups, 
		{
			title = L["Trivial"],
			filterType = "difficulty",
			filterIndex = "trivial",
		});
	
	
	tinsert(MrTraderSkillWindow.filters, 
		{
			title = L["Favorites"],
			filterType = "favorite",
			filterIndex = "__all",
			subgroups = {},
		});
			
	local tradeskillSpellID = MrTrader:MatchPartialCraftSkillName(GetTradeSkillLine());
	local favorites = MrTrader:GetFavoritesForSkill(tradeskillSpellID);
	if( favorites and MrTrader:TableSize(favorites) > 0 ) then
		parent = MrTraderSkillWindow.filters[5];
		for categoryName, _ in pairs(favorites) do
			tinsert(parent.subgroups,
				{
					title = categoryName,
					filterType = "favorite",
					filterIndex = categoryName,
				})
		end
	end
	
	if( MrTraderSkillWindow.filterCategory ~= nil ) then
		local filterCategory = nil;
		local filterSelection = nil;
	
		for i=1, getn(MrTraderSkillWindow.filters) do
			if( MrTraderSkillWindow.filterCategory.title == MrTraderSkillWindow.filters[i].title and
				MrTraderSkillWindow.filterCategory.filterIndex == MrTraderSkillWindow.filters[i].filterIndex ) then
				filterCategory = MrTraderSkillWindow.filters[i];
				if( filterCategory.filterType ~= nil ) then
					filterSelection = MrTraderSkillWindow.filters[i];
				end
				
				if( MrTraderSkillWindow.filterSelection ~= nil ) then
					for j=1, getn(filterCategory.subgroups) do
						if( filterCategory.subgroups[j].title == MrTraderSkillWindow.filterSelection.title and
							filterCategory.subgroups[j].filterIndex == MrTraderSkillWindow.filterSelection.filterIndex ) then
							filterSelection = filterCategory.subgroups[j];
							break;
						end
					end
				end
				break;
			end
		end
		
		MrTraderSkillWindow.filterCategory = filterCategory;
		MrTraderSkillWindow.filterSelection = filterSelection;
	end
	
	if( MrTraderSkillWindow.filterCategory == nil ) then 
		MrTraderSkillWindow.filterCategory = MrTraderSkillWindow.filters[1];
		MrTraderSkillWindow.filterSelection = MrTraderSkillWindow.filters[1];
	end
end

function MrTraderSkillWindow.Close()
	HideUIPanel(MrTrader_SkillFrame);
end

function MrTraderSkillWindow.Update()
	if( not MrTrader_SkillFrame:IsVisible() ) then
		return;
	end

	-- Update Scan
	MrTraderSkillWindow.ParseSkillList();
	
	-- Update Display
	MrTraderSkillWindow.UpdateHeader();
	MrTraderSkillWindow.UpdateCategories();
	MrTraderSkillWindow.UpdateSkillList();
	MrTraderSkillWindow.SetSkillSelection(MrTraderSkillWindow.selectedSkillButton, MrTraderSkillWindow.selectedSkill);
end

function MrTraderSkillWindow.UpdateHeader()
	local name, rank, maxRank = GetTradeSkillLine();

	-- Title
	MrTrader_SkillFrameTitleText:SetFormattedText(TRADE_SKILL_TITLE, name);
	
	-- Link Button
	if ( IsTradeSkillLinked() ) then
		MrTrader_SkillLinkButton:Hide();
	else
		if ( GetTradeSkillListLink() ) then
			MrTrader_SkillLinkButton:Show();
		else
			MrTrader_SkillLinkButton:Hide();
		end
	end
	
	-- Rank Bar
	MrTrader_SkillRankFrame:SetStatusBarColor(0.0, 0.0, 1.0, 0.5);
	MrTrader_SkillRankFrameBackground:SetVertexColor(0.0, 0.0, 0.75, 0.5);
	MrTrader_SkillRankFrame:SetMinMaxValues(0, maxRank);
	MrTrader_SkillRankFrame:SetValue(rank);
	MrTrader_SkillRankFrameSkillRank:SetText(rank.."/"..maxRank);
	
	-- Search Bar
	if ( rank < 75 ) and ( not IsTradeSkillLinked() ) then
		MrTrader_SkillFrameEditBox:Hide();
		SetTradeSkillItemNameFilter("");	--In case they are switching from an inspect WITH a filter directly to their own without.
	else
		MrTrader_SkillFrameEditBox:Show();
	end
end

function MrTraderSkillWindow.ParseSkillList()
	MrTraderSkillWindow.scannedSkillLists = {};
	MrTraderSkillWindow.scannedSkillLists["difficulty"] = {};
	MrTraderSkillWindow.scannedSkillLists["difficulty"]["optimal"] = {};
	MrTraderSkillWindow.scannedSkillLists["difficulty"]["medium"] = {};
	MrTraderSkillWindow.scannedSkillLists["difficulty"]["easy"] = {};
	MrTraderSkillWindow.scannedSkillLists["difficulty"]["trivial"] = {};
	MrTraderSkillWindow.scannedSkillLists["favorites"] = {};
	MrTraderSkillWindow.scannedSkillLists["favorites"]["__all"] = {};
	
	-- Setup for favorites
	local skillName = GetTradeSkillLine();
	local tradeskillSpellID = MrTrader:MatchPartialCraftSkillName(skillName);
	local favorites = MrTrader:GetFavoritesForSkill(tradeskillSpellID);

	if( favorites ) then
		for categoryName, _ in pairs(favorites) do
			MrTraderSkillWindow.scannedSkillLists["favorites"][categoryName] = {};
		end
	end

	-- Iterate through the filtered list and sort
	local serverCount = GetNumTradeSkills();
	for i=1, serverCount do
		local _, skillType = GetTradeSkillInfo(i);
		if( skillType ~= "header" ) then
			-- Add the item to a difficulty list
			tinsert(MrTraderSkillWindow.scannedSkillLists["difficulty"][skillType], i);
			
			-- Do a favorites check
			if( favorites ) then
				local inFavorites = false;
				local skillSpellID = MrTrader:GetSpellIDForRecipe(GetTradeSkillRecipeLink(i));
				for categoryName, categoryItems in pairs(favorites) do
					if( categoryItems[skillSpellID] == true ) then
						tinsert(MrTraderSkillWindow.scannedSkillLists["favorites"][categoryName], i);
						inFavorites = true;
					end
				end
			
				if( inFavorites == true ) then
					tinsert(MrTraderSkillWindow.scannedSkillLists["favorites"]["__all"], i);
				end
			end
		end
	end
end

function MrTraderSkillWindow.GetSkillList()
	local skillList = {};
	local serverCount = GetNumTradeSkills();
	
	-- Get the currently selected skill list if we have one
	if( MrTraderSkillWindow.filterSelection ~= nil ) then
		if( MrTraderSkillWindow.filterSelection.filterType == "difficulty" ) then
			return MrTraderSkillWindow.scannedSkillLists["difficulty"][MrTraderSkillWindow.filterSelection.filterIndex];
		elseif( MrTraderSkillWindow.filterSelection.filterType == "favorite" ) then
			return MrTraderSkillWindow.scannedSkillLists["favorites"][MrTraderSkillWindow.filterSelection.filterIndex];
		end
	end
	
	-- Otherwise, scan the current state of the server
	for i=1, serverCount do
		local _, skillType = GetTradeSkillInfo(i);
		if( skillType ~= "header" ) then
			tinsert(skillList, i);
		end
	end
	
	return skillList;
end

function MrTraderSkillWindow.UpdateSkillList()
	local skillList = MrTraderSkillWindow.GetSkillList();
	local numTradeSkills = getn(skillList);

	-- If we have no filter of any kind, then we need to empty the skill list
	if( MrTraderSkillWindow.filterSelection == nil ) then
		numTradeSkills = 0;
	end

	-- ScrollFrame update
	FauxScrollFrame_Update(MrTrader_SkillListScrollFrame, numTradeSkills, TRADE_SKILLS_DISPLAYED, MRTRADER_TRADE_SKILL_HEIGHT, nil, nil, nil, MrTrader_SkillHighlightFrame, 293, 316 );
	local skillIndex = FauxScrollFrame_GetOffset(MrTrader_SkillListScrollFrame);
	local skillHighlighted = false;
	
	for i=1, TRADE_SKILLS_DISPLAYED do
		skillIndex = skillIndex + 1;
			
		skillButton = _G["MrTrader_SkillButton"..i];
		skillButtonText = _G["MrTrader_SkillButton"..i.."Text"];
		skillButtonCount = _G["MrTrader_SkillButton"..i.."Count"];
		if( skillIndex <= numTradeSkills ) then
			local skillName, skillType, numAvailable = GetTradeSkillInfo(skillList[skillIndex]);
			skillButton:SetWidth(323);
			
			local color = TradeSkillTypeColor[skillType];
			if( color ) then
				skillButton:SetNormalFontObject(color.font);
				skillButtonCount:SetVertexColor(color.r, color.g, color.b);
				skillButton.r = color.r;
				skillButton.g = color.g;
				skillButton.b = color.b;
			end
			
			local skillNamePrefix = "";
			local skillNameSuffix = "";
			if ( ENABLE_COLORBLIND_MODE == "1" ) then
				skillNamePrefix = TradeSkillTypePrefix[skillType] or " ";
			end
			
			skillButton:SetID(skillList[skillIndex]);
			skillButton:Show();
			
			skillButton:SetNormalTexture("");
			_G["MrTrader_SkillButton"..i.."Highlight"]:SetTexture("");
			
			if( numAvailable > 0 ) then
				skillButtonCount:SetText("["..numAvailable.."]");
				MrTrader_SkillFrameDummyString:SetText(skillName);
				nameWidth = MrTrader_SkillFrameDummyString:GetWidth();
				countWidth = skillButtonCount:GetWidth();
				skillButtonText:SetText(skillName);
				if ( nameWidth + 2 + countWidth > TRADE_SKILL_TEXT_WIDTH ) then
					skillButtonText:SetWidth(TRADE_SKILL_TEXT_WIDTH-2-countWidth);
				else
					skillButtonText:SetWidth(0);
				end
			else
				skillButton:SetText(skillNamePrefix..skillName..skillNameSuffix);
				skillButtonText:SetWidth(TRADE_SKILL_TEXT_WIDTH);
				skillButtonCount:SetText(skillCountPrefix);	
			end
			
			-- Place the highlight and lock the highlight state
			if ( MrTraderSkillWindow.selectedSkill == skillList[skillIndex] ) then
				MrTrader_SkillHighlightFrame:SetPoint("TOPLEFT", "MrTrader_SkillButton"..i, "TOPLEFT", 0, 0);
				MrTrader_SkillHighlightFrame:Show();
				skillButtonCount:SetVertexColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
				skillButton:LockHighlight();
				skillButton.isHighlighted = true;
				skillHighlighted = true;
			else
				skillButton:UnlockHighlight();
				skillButton.isHighlighted = false;
			end
		else
			skillButton:Hide();
		end
	end
	
	if( skillHighlighted == false ) then
		MrTrader_SkillHighlightFrame:Hide();
	end
end

function MrTraderSkillWindow.UpdateCategories()
	local subClasses = {GetTradeSkillSubClasses()};
	local numCategories = getn(subClasses);

	if(MrTraderSkillWindow.filters == nil) then
		return;
	end
	
	-- Initialize the list of open filters
	OPEN_FILTER_LIST = {};

	local numGroups = getn(MrTraderSkillWindow.filters);
	for i=1, numGroups do
		local item = MrTraderSkillWindow.filters[i];
		if( MrTraderSkillWindow.filterCategory == item ) then
			tinsert(OPEN_FILTER_LIST, {item.title, "class", item, true});
			local numSubgroups = getn(item.subgroups);
			for j=1, numSubgroups do
				local subgroup = item.subgroups[j];
				if( MrTraderSkillWindow.filterSelection == subgroup ) then
					tinsert(OPEN_FILTER_LIST, {subgroup.title, "subclass", subgroup, true});			
				else
					tinsert(OPEN_FILTER_LIST, {subgroup.title, "subclass", subgroup, nil});							
				end
			end
		else
			tinsert(OPEN_FILTER_LIST, {item.title, "class", item, nil});			
		end
	end
	
	-- ScrollFrame update
	FauxScrollFrame_Update(MrTrader_SkillFilterScrollFrame, getn(OPEN_FILTER_LIST), NUM_FILTERS_TO_DISPLAY, MRTRADER_TRADE_SKILL_FILTER_HEIGHT);
	local filterIndex = FauxScrollFrame_GetOffset(MrTrader_SkillFilterScrollFrame) ;
	
	-- Display the list of open filters
	local button, index, info, isLast;
	index = FauxScrollFrame_GetOffset(MrTrader_SkillFilterScrollFrame);
	for i=1, NUM_FILTERS_TO_DISPLAY do
		button = _G["MrTrader_SkillFilterButton"..i];
		if ( getn(OPEN_FILTER_LIST) > NUM_FILTERS_TO_DISPLAY ) then
			button:SetWidth(136);
		else
			button:SetWidth(156);
		end

		index = index + 1;
		if ( index <= getn(OPEN_FILTER_LIST) ) then
			info = OPEN_FILTER_LIST[index];
			if ( info ) then
				MrTraderSkillWindow.FilterButton_SetType(button, info[2], info[1], info[5]);
				button.filterRef = info[3];
				if ( info[4] ) then
					button:LockHighlight();
				else
					button:UnlockHighlight();
				end
				button:Show();
			end
		else
			button:Hide();
		end
	end
end

function MrTraderSkillWindow.FilterButton_SetType(button, type, text, isLast)
	local normalText = _G[button:GetName().."NormalText"];
	local normalTexture = _G[button:GetName().."NormalTexture"];
	local line = _G[button:GetName().."Lines"];
	
	line:Hide();
	if ( type == "class" ) then
		button:SetText(text);
		normalText:SetPoint("LEFT", button, "LEFT", 4, 0);
		normalTexture:SetAlpha(1.0);	
	elseif ( type == "subclass" ) then
		button:SetText(HIGHLIGHT_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE);
		normalText:SetPoint("LEFT", button, "LEFT", 12, 0);
		normalTexture:SetAlpha(0.4);
	end
	button.type = type; 
end

function MrTraderSkillWindow.OnSkillSelect(skill, button)
	-- Do something here... maybe
	if ( button == "LeftButton" ) then
		MrTraderSkillWindow.SetSkillSelection(skill, skill:GetID());
		MrTraderSkillWindow.Update();
	elseif ( button == "RightButton" ) then
		MrTraderSkillWindow:DisplaySkillContext(skill, skill:GetID());
	end
end

function MrTraderSkillWindow.SetSkillSelection(skillButton, skillID)
	local skillName, skillType, numAvailable, isExpanded, altVerb = GetTradeSkillInfo(skillID);
	local craftable = 1;
	if( not skillName ) then
		craftable = nil;
	end
	
	MrTraderSkillWindow.selectedSkillButton = skillButton;
	MrTraderSkillWindow.selectedSkill = skillID;
	
	if ( MrTraderSkillWindow.selectedSkill > GetNumTradeSkills() ) then
		return;
	end
	
	-- Hide inappropriate controls when no skill is selected
	if ( MrTraderSkillWindow.selectedSkill == 0 ) then
		MrTrader_SkillCreateButton:Hide();
		MrTrader_SkillCreateAllButton:Hide();
		MrTrader_SkillDecrementButton:Hide();
		MrTrader_SkillInputBox:Hide();
		MrTrader_SkillIncrementButton:Hide();
		MrTrader_DetailScrollFrame:Hide();
		return;
	else
		MrTrader_DetailScrollFrame:Show();
	end
	
	local color = TradeSkillTypeColor[skillType];
	if ( color ) then
		MrTrader_SkillHighlight:SetVertexColor(color.r, color.g, color.b);
	end
	
	-- Name and Cooldown
	MrTrader_SkillName:SetText(skillName);
	if ( GetTradeSkillCooldown(skillID) ) then
		MrTrader_SkillCooldown:SetText(COOLDOWN_REMAINING.." "..SecondsToTime(GetTradeSkillCooldown(skillID)));
	else
		MrTrader_SkillCooldown:SetText("");
	end
	
	-- Icon and Number Made
	MrTrader_SkillIcon:SetNormalTexture(GetTradeSkillIcon(skillID));
	local minMade, maxMade = GetTradeSkillNumMade(skillID);
	if ( maxMade > 1 ) then
		if ( minMade == maxMade ) then
			MrTrader_SkillIconCount:SetText(minMade);
		else
			MrTrader_SkillIconCount:SetText(minMade.."-"..maxMade);
		end
		if ( MrTrader_SkillIconCount:GetWidth() > 39 ) then
			MrTrader_SkillIconCount:SetText("~"..floor((minMade + maxMade)/2));
		end
	else
		MrTrader_SkillIconCount:SetText("");
	end
	
	-- Reagents
	local numReagents = GetTradeSkillNumReagents(skillID);
	if(numReagents > 0) then
		MrTrader_SkillReagentLabel:Show();
	else
		MrTrader_SkillReagentLabel:Hide();
	end
	for i=1, numReagents, 1 do
		local reagentName, reagentTexture, reagentCount, playerReagentCount = GetTradeSkillReagentInfo(skillID, i);
		local reagent = _G["MrTrader_SkillReagent"..i]
		local name = _G["MrTrader_SkillReagent"..i.."Name"];
		local count = _G["MrTrader_SkillReagent"..i.."Count"];
		if ( not reagentName or not reagentTexture ) then
			reagent:Hide();
		else
			reagent:Show();
			SetItemButtonTexture(reagent, reagentTexture);
			name:SetText(reagentName);
			-- Grayout items
			if ( playerReagentCount < reagentCount ) then
				SetItemButtonTextureVertexColor(reagent, 0.5, 0.5, 0.5);
				name:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
				craftable = nil;
			else
				SetItemButtonTextureVertexColor(reagent, 1.0, 1.0, 1.0);
				name:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			end
			if ( playerReagentCount >= 100 ) then
				playerReagentCount = "*";
			end
			count:SetText(playerReagentCount.." /"..reagentCount);
		end
	end
	-- Place reagent label
	local reagentToAnchorTo = numReagents;
	if ( (numReagents > 0) and (mod(numReagents, 2) == 0) ) then
		reagentToAnchorTo = reagentToAnchorTo - 1;
	end
	-- Hide unused reagents
	for i=numReagents + 1, MAX_TRADE_SKILL_REAGENTS, 1 do
		_G["MrTrader_SkillReagent"..i]:Hide();
	end
	
	-- Tools
	local spellFocus = BuildColoredListString(GetTradeSkillTools(skillID));
	if ( spellFocus ) then
		MrTrader_SkillRequirementLabel:Show();
		MrTrader_SkillRequirementText:SetText(spellFocus);
	else
		MrTrader_SkillRequirementLabel:Hide();
		MrTrader_SkillRequirementText:SetText("");
	end
	
	-- Description
	if ( GetTradeSkillDescription(skillID) ) then
		MrTrader_SkillDescription:SetText(GetTradeSkillDescription(skillID))
		MrTrader_SkillReagentLabel:SetPoint("TOPLEFT", "MrTrader_SkillDescription", "BOTTOMLEFT", 0, -10);
	else
		MrTrader_SkillDescription:SetText(" ");
		MrTrader_SkillReagentLabel:SetPoint("TOPLEFT", "MrTrader_SkillDescription", "TOPLEFT", 0, 0);
	end
	
	-- Reset the number of items to be created
	MrTrader_SkillInputBox:SetNumber(GetTradeskillRepeatCount());
	
	--Hide inapplicable buttons if we are inspecting. Otherwise show them
	if ( IsTradeSkillLinked() ) then
		MrTrader_SkillCreateButton:Hide();
		MrTrader_SkillCreateAllButton:Hide();
		MrTrader_SkillDecrementButton:Hide();
		MrTrader_SkillInputBox:Hide();
		MrTrader_SkillIncrementButton:Hide();
	else
		--Change button names and show/hide them depending on if this tradeskill creates an item or casts something
		if ( not altVerb ) then
			--Its an item with 'Create'
			MrTrader_SkillCreateAllButton:Show();
			MrTrader_SkillDecrementButton:Show();
			MrTrader_SkillInputBox:Show();
			MrTrader_SkillIncrementButton:Show();
		else
			--Its something else
			MrTrader_SkillCreateAllButton:Hide();
			MrTrader_SkillDecrementButton:Hide();
			MrTrader_SkillInputBox:Hide();
			MrTrader_SkillIncrementButton:Hide();
		end
		
		MrTrader_SkillCreateButton:SetText(altVerb or CREATE);
		MrTrader_SkillCreateButton:Show();
    end

	MrTraderSkillWindow.numAvailable = numAvailable;
end

function MrTraderSkillWindow.OnFilterSelect(filter, button)
	-- Do Something here... maybe
	if( button == "LeftButton" ) then
		if( filter.filterRef == nil ) then
			return;
		end
		
		local item = filter.filterRef;
	
		MrTraderSkillWindow.filterSelection = nil;
		-- Close everything up if they clicked on the filter category.
		if( item == MrTraderSkillWindow.filterCategory ) then
			MrTraderSkillWindow.filterCategory = nil;
		else
			if( item.subgroups ~= nil ) then
				MrTraderSkillWindow.filterCategory = item;
			end

			if( item.filterType ~= nil ) then
				MrTraderSkillWindow.filterSelection = item;
			end

			local subclassFilter = 0;
			local invslotFilter = 0;

			if( MrTraderSkillWindow.filterSelection ~= nil ) then
				if( MrTraderSkillWindow.filterSelection.filterType == "invslot" ) then
					invslotFilter = MrTraderSkillWindow.filterSelection.filterIndex;
				elseif( MrTraderSkillWindow.filterSelection.filterType == "category" ) then
					subclassFilter = MrTraderSkillWindow.filterSelection.filterIndex;
				end
			end

			SetTradeSkillSubClassFilter(subclassFilter, 1, 1);
			SetTradeSkillInvSlotFilter(invslotFilter, 1, 1)
		end
		
		MrTraderSkillWindow.SetSkillSelection(nil, 0);
		MrTraderSkillWindow.Update();
	end
end

function MrTraderSkillWindow.OnEnterSkillIcon(icon)
	if ( MrTraderSkillWindow.selectedSkill ~= 0 ) then
		GameTooltip:SetOwner(icon, "ANCHOR_RIGHT");
		GameTooltip:SetTradeSkillItem(MrTraderSkillWindow.selectedSkill);
	end
	CursorUpdate(icon);
end

function MrTraderSkillWindow_Update()
	MrTraderSkillWindow.ParseSkillList();
	MrTraderSkillWindow.UpdateSkillList();
end

function MrTraderSkillWindow_UpdateFilters()
	MrTraderSkillWindow.UpdateCategories();
end

function MrTraderSkillWindow_UpdateHeaders()
	MrTraderSkillWindow.UpdateHeader();
end

function MrTraderSkillWindow.OnShow()
--	MrTraderSkillWindow.HideBlizzardFrame();
end

function MrTraderSkillWindow.OnHide()
	MrTrader_NewCategoryFrame:Hide();
end

function MrTraderSkillWindow_OnEnterSkillIcon(icon)
	MrTraderSkillWindow.OnEnterSkillIcon(icon);
end

function MrTraderSkillWindow_OnSkillClick(self, button)
	if ( IsShiftKeyDown() ) then
		HandleModifiedItemClick(GetTradeSkillRecipeLink(self:GetID()));
	else
		MrTraderSkillWindow.OnSkillSelect(self, button);
	end
end

function MrTraderSkillWindow_OnSearchTextChange(field)
	TradeSkillFilter_OnTextChanged(field);
	
	MrTraderSkillWindow_Update();
end

function MrTraderSkillWindow.OnDecrementClick()
	if ( MrTrader_SkillInputBox:GetNumber() > 0 ) then
		MrTrader_SkillInputBox:SetNumber(MrTrader_SkillInputBox:GetNumber() - 1);
	end
end

function MrTraderSkillWindow.OnIncrementClick()
	if ( MrTrader_SkillInputBox:GetNumber() < 100 ) then
		MrTrader_SkillInputBox:SetNumber(MrTrader_SkillInputBox:GetNumber() + 1);
	end
end